/*********************************************
  Autor: Marlon Nardi Walendorff
  Projeto: Compressor de ar feito com Arduino - Compressino
  Detalhes do projeto:https://marlonnardi.com/2022/07/13/compressor-de-ar-feito-em-casa-de-baixo-custo-com-arduino-compressino/
  /**********************************************/

//==================== Inclusão de Bibliotecas =================//
#include <Adafruit_PCD8544.h> 
#include <Adafruit_GFX.h>
#include <RotaryEncoder.h>
#include <TimerOne.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <EEPROM.h>

//==================== Mapeamento de Hardware ==================//
#define pin_Encoder_DT 2
#define pin_Encoder_CLK 3
#define pin_Encoder_SW 4

#define pin_Display_CLK_SCK 5
#define pin_Display_DIN_MOSI 6
#define pin_Display_DC 7 
#define pin_Display_CE_CS_SCE 8
#define pin_Display_RST 9
#define pin_Display_Light 10

#define pin_Buzzer 11

#define pin_Relay 12

#define pin_Temp_Sensor 13

#define pin_Pressure_Sensor A0
#define pin_AC_Detection A1


//==================== Instanciando Objetos ==================//
Adafruit_PCD8544 display = Adafruit_PCD8544(pin_Display_CLK_SCK, pin_Display_DIN_MOSI, pin_Display_DC, pin_Display_CE_CS_SCE, pin_Display_RST);
RotaryEncoder EncoderOne(pin_Encoder_CLK, pin_Encoder_DT);
OneWire oneWire(pin_Temp_Sensor);  

// Passa uma referência oneWire para a biblioteca DallasTemperature
DallasTemperature TempSensor(&oneWire);

//==================== Variáveis Globais ==================//
extern byte menu_Position;
extern uint8_t max_pressure;  
extern uint8_t min_pressure; 
extern uint8_t current_motor_temperature; 
extern uint16_t current_pressure_sensor_PSI;

extern volatile uint8_t active_time1;
extern volatile uint8_t active_time2;

byte aux4 = 0;

volatile uint16_t time1_in_tenths_of_a_seconds = 0;
volatile uint16_t time2_in_tenths_of_a_seconds = 0;
volatile uint16_t time3_in_tenths_of_a_seconds = 0;
volatile uint16_t time4_in_tenths_of_a_seconds_motor_on = 0;
volatile uint16_t time5_in_tenths_of_a_seconds_motor_off = 0;
volatile uint16_t time6_in_tenths_of_a_seconds_verify_pressure_sensor = 0;
volatile uint16_t time7_in_tenths_of_a_seconds_read_pressure_sensor = 0;

void setup()
{
  Serial.begin(9600); 
  pinMode(pin_Display_Light, OUTPUT);
  pinMode(pin_Buzzer, OUTPUT); 
  pinMode(pin_Relay, OUTPUT); 
  digitalWrite(pin_Relay,HIGH);
  
  display.begin(); 
  display.setContrast(25);
  display.clearDisplay(); 
  display.display();

  TempSensor.begin();

  //----------------- Interrupção Externa ------------------------//
  /* Vincula duas interrupções externas no pino 2 e 3 nas funções ISR0 e ISR1
  para garantir que o encoder sempre seja lido com prioridade. */
  attachInterrupt(digitalPinToInterrupt(2), ISR0, CHANGE);
  attachInterrupt(digitalPinToInterrupt(3), ISR1, CHANGE);

  //----------------- Interrupção Interna ------------------------//
   Timer1.initialize(100000); // set a timer of length 100000 microseconds (or 0.1 sec - or 10Hz => the led will blink 5 times, 5 cycles of on-and-off, per second)
   Timer1.attachInterrupt(timerIsr); // attach the service routine here 
  
  //----------------- Dados salvos na EEPROM ------------------------//
  EEPROM.get(0, max_pressure);
  EEPROM.put(0, max_pressure);


  EEPROM.get(10, min_pressure);
  EEPROM.put(10, min_pressure);

  Serial.println(analogRead(pin_Pressure_Sensor));
  

}//------EndSetup

void loop()
{
eeprom();
relay();




//------------------- Faz a primeira leitura do Sensor de Temperatura --------------//
if(aux4 == 0)
{
 readTempSensor();
 aux4 = 1; 
}


//------------------- Lê o sensor de temperatura de tempo em tempo --------------//
if(time3_in_tenths_of_a_seconds >= 100)
{ 
readTempSensor();
time3_in_tenths_of_a_seconds = 0;
}

readEncoderAndMenu();
// max_pressure,min_pressure,current_pressure,temperature_motor, cursor_position
refreshDisplay(max_pressure,min_pressure,current_pressure_sensor_PSI,current_motor_temperature,menu_Position);
}//------EndLoop


//----------------- Interrupção Interna ------------------------//
void timerIsr()
{
buzzerAlert();
  
 if(active_time1){
time1_in_tenths_of_a_seconds++; 
}else{
time1_in_tenths_of_a_seconds = 0; 
}


 if(active_time2){
time2_in_tenths_of_a_seconds++; 
}else{
time2_in_tenths_of_a_seconds = 0; 
}


time3_in_tenths_of_a_seconds++;


if(!digitalRead(pin_Relay) && time4_in_tenths_of_a_seconds_motor_on < 55){
time4_in_tenths_of_a_seconds_motor_on++; 
}

if(!digitalRead(pin_Relay)){
time5_in_tenths_of_a_seconds_motor_off = 0;
}


if(digitalRead(pin_Relay) && time5_in_tenths_of_a_seconds_motor_off < 55){
time5_in_tenths_of_a_seconds_motor_off++; 
}

if(digitalRead(pin_Relay)){
time4_in_tenths_of_a_seconds_motor_on = 0;
}

if(digitalRead(pin_Relay) == 0 && time6_in_tenths_of_a_seconds_verify_pressure_sensor < 520){
 time6_in_tenths_of_a_seconds_verify_pressure_sensor++;  
}else{
time6_in_tenths_of_a_seconds_verify_pressure_sensor = 0;  
}


if(time7_in_tenths_of_a_seconds_read_pressure_sensor < 15){
 time7_in_tenths_of_a_seconds_read_pressure_sensor++;  
}else{
readPressure();  
time7_in_tenths_of_a_seconds_read_pressure_sensor = 0;  
}


checkVoltageMotor();





 if(time1_in_tenths_of_a_seconds >= 1 && time1_in_tenths_of_a_seconds <2) digitalWrite(pin_Buzzer, HIGH);
 if(time1_in_tenths_of_a_seconds >= 2 && time1_in_tenths_of_a_seconds <3) digitalWrite(pin_Buzzer, LOW); 
 if(time1_in_tenths_of_a_seconds >= 3 && time1_in_tenths_of_a_seconds <4) digitalWrite(pin_Buzzer, HIGH);
 if(time1_in_tenths_of_a_seconds >= 4 && time1_in_tenths_of_a_seconds <5) digitalWrite(pin_Buzzer, LOW);
 if(time1_in_tenths_of_a_seconds >= 5 && time1_in_tenths_of_a_seconds <6) digitalWrite(pin_Buzzer, HIGH);
 if(time1_in_tenths_of_a_seconds >= 6) digitalWrite(pin_Buzzer, LOW);

  if(time2_in_tenths_of_a_seconds >= 1 && time2_in_tenths_of_a_seconds < 20) digitalWrite(pin_Buzzer, HIGH);
 if(time2_in_tenths_of_a_seconds >= 20 ) digitalWrite(pin_Buzzer, LOW); 


}//----- EndTimerISR
